@@ -20,6 +20,7 @@ from Foundation import (
     NSData,
     NSError,
 )
+from CoreBluetooth import CBCharacteristicWriteWithResponse, CBCharacteristicWriteWithoutResponse
 
 from bleak.exc import BleakError
 
@@ -159,7 +160,8 @@ class PeripheralDelegate(NSObject):
         self.peripheral.writeValue_forCharacteristic_type_(
             value, characteristic, response
         )
-        if response:
+
+        if response == CBCharacteristicWriteWithResponse:
             await event.wait()
 
         return True
